;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xfe2bac2b07d59a05a7c57fcebf58c794 . "./code.cg2js/cg-status-bar.fasl")
(#x1252d98e71eb678252ebd0b391284916 . "./code.cg2js/cg-hotspot.fasl")
(#x97407c20700acf37ccb76088a0576d4c . "./code.cg2js/cg-toolbar.fasl")
(#x260d4109adff56ce594d6c23f787291b . "./code.cg2js/cg-clipboard-stack.fasl")
(#xdd832ad3e29537f0ddb4be6ae9bafef6 . "./code.cg2js/cg-tooltip.fasl")
(#x0bf85b52508b530624b696830c45311c . "./code.cg2js/cg-calendar.fasl")
(#x46ad5902da6402a2cfdc4b99987dfa97 . "./code.cg2js/cg-item-list.fasl")
(#xff816908614b2a13d098a427f6ae6152 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xb50f9bd2fd3ad81d53df47c2c2e176e5 . "./code.cg2js/cg-trackbar.fasl")
(#x60f80dcb1c8080894d80ca143a410e5e . "./code.cg2js/cg-class-grid.fasl")
(#xbb636d5267fd64c2c18ea7dbec5c816d . "./code.cg2js/cg-dialog-item.fasl")
(#x5360af7150f61fa848630bc803964aea . "./code.cg2js/cg-comtab.fasl")
(#x2d743a53e902ee0ea6bcb233a73806b3 . "./code.cg2js/cg-lisp-text.fasl")
(#xe085ad98c17f29403eda907a205a5176 . "./code.cg2js/cg-button.fasl")
(#xb226f5a24e4371ad50cbd670fa81bf30 . "./code.cg2js/cg-printing.fasl")
(#x6adbb0044f1a9d805a46d6c240e8286f . "./code.cg2js/cg-pixmap-printing.fasl")
(#x16bc396ab5f4e081badedb1591f148b1 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x5ac9bc61e9e5f76d5a57d6340ce88f91 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xf4c1f51dd5dccb5e3ec4a10d898245df . "./code.cg2js/cg-selected-object.fasl")
(#x3587644b2b0dd1485d4d1cac69e62b6b . "./code.cg2js/cg-text-or-combo.fasl")
(#x49af8a6f0a62bafa72d3788f6a71ca08 . "./code.cg2js/cg-graphics-context.fasl")
(#x3b902fdaeddb5b8370174887fe63ccf9 . "./code.cg2js/cg-edit-in-place.fasl")
(#x0f43789d1511bb5f6c6978530e34f864 . "./code.cg2js/cg-string-dialog.fasl")
(#x2233599b27ee609062a6ec6fca0d368e . "./code.cg2js/cg-list-view.fasl")
(#xb52f2f91de040f53d843dc171d0af8f2 . "./code.cg2js/cg-intersect.fasl")
(#x73590b799a803383f0978ebccf346f73 . "./code.cg2js/cg-text-widget.fasl")
(#xe90b6ca59532f69aae8a4ed4f597728b . "./code.cg2js/cg-directory-dialog.fasl")
(#x75c115b0ea19845f5372e71c9542083b . "./code.cg2js/cg-file-dialog.fasl")
(#x99d7605d32ab0dcd7046ca4479322193 . "./code.cg2js/cg-dde-utils.fasl")
(#xaf8516d53d6a0d504a11660def618fb5 . "./code.cg2js/cg-nodes.fasl")
(#x2017fe344682987f84735d1b1d7dda12 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x4f088ba58b3c041064f862b2eb722f7d . "./code.cg2js/cg-up-down-control.fasl")
(#x76b2962070db22cb508e4185f6347ec4 . "./code.cg2js/cg-get-position.fasl")
(#xd54862a7b37d74a7b9e73e9441b293cd . "./code.cg2js/cg-toggling-widget.fasl")
(#xd4712f73cf8d2f0684351858e505f650 . "./code.cg2js/cg-acache.fasl")
(#xec4aad1e57fde060bef1d5d6de3ab9fb . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x61c159608bc4b3ba143145ecf18352dd . "./code.cg2js/cg-base.fasl")
(#x392d2b6025b616cca449aebfe75478a3 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x0c9fa2b2cd30ef9d5c92a3d951dcf128 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x59beae448dbcd0ae2923041d685b0081 . "./code.cg2js/cg-property.fasl")
(#xfb91cd57dd55a2886353ad107696a658 . "./code.cg2js/cg-editable-text.fasl")
(#xdc8195fa404e0f84a4873c461a3b17fa . "./code.cg2js/cg-lamp.fasl")
(#xa195581422cc74d1afe4c9aa7006c0fa . "./code.cg2js/cg-progress-indicator.fasl")
(#x3f35488a2f73383a2a9f8bbe9d499f78 . "./code.cg2js/cg-chart-widget.fasl")
(#x514aced070930ca1790450d42d55cbd8 . "./code.cg2js/cg-common-control.fasl")
(#x315b0018a4aacd591a3869ec548681db . "./code.cg2js/cg-font-dialog.fasl")
(#x8d56c3a3575b9bc9c838c6a28c33f19c . "./code.cg2js/cg-gesture-emulation.fasl")
(#x00f1eb63dc2f4dd5c0d7956a8b9f8d8d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xe3ca8d65d376790bd99c9b6ae79cb24a . "./code.cg2js/cg-os-window.fasl")
(#xec34cbb18a049e9d47ab032a7659fd89 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x799081adb1d157050d65233e7b45571e . "./code.cg2js/cg-pixmap.fasl")
(#x98f7fdfc29be30a76a060589c815ac13 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x5f11b67434ced4c54053aaae77062370 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xca2424b117af02d035dc3f997347c203 . "./code.cg2js/cg-scaling-stream.fasl")
(#x6892777f8c2f3662f7df100b45e602e1 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x9af9738c188119c3f314ecbd35d5509c . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x260729623c36fd897f7050a91597811b . "./code.cg2js/cg-timer.fasl")
(#x4e5b9019bfb516372d9f8990260f34e4 . "./code.cg2js/cg-scroll-bar.fasl")
(#x7021b0c21750d0e2ca5003d81b135860 . "./code.cg2js/cg-caret.fasl")
(#x8ece539326353d53204e14a5da6b758c . "./code.cg2js/cg-template-string.fasl")
(#x03da85070cc3c5aec3c207080323d1a6 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x6f126a236c7a789e2dd0dfb3482f2e6d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xdcdd00d2d59f8fc1737a31d21b017bb9 . "./code.cg2js/cg-combo-box.fasl")
(#x8292234a8e068905be5ef9c9a9ddd5f3 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xb7d36aa1192eb37a3e247b805ca4ddb2 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x5a31808a35871528cb595deeacc9f6dc . "./code.cg2js/cg-menu-tooltip.fasl")
(#x6afe5a6f3c422b3b3654d604aa893079 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x8b9ada6744d986c4a8bf37e51f7e2e2e . "./code.cg2js/cg-player.fasl")
(#x313e1bcac6cb0c2ea58f49993cda7deb . "./code.cg2js/cg-outline.fasl")
(#x4f343cd2b5a0f64509b3259ed43506c5 . "./code.cg2js/cg-class-support.fasl")
(#x782932b10a3e3daa84703096876e7d8a . "./code.cg2js/cg-lettered-menu.fasl")
(#x4317da597d2b6c433b57f5f6c4df1d71 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd3bcfc86953f63189216705514f6aee1 . "./code.cg2js/cg-fill-texture.fasl")
(#xcbbab49ace9dd993c649ee8d7352ff2d . "./code.cg2js/cg-lisp-widget.fasl")
(#x777b3b5f3b292061dbb1705829161e90 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x660aa925ba864c2ae6dfaa9331c6a1c0 . "./code.cg2js/cg-clipboard.fasl")
(#x3121df6fea0794c5cc14085fe0c4692c . "./code.cg2js/cg-wrap-string.fasl")
(#x8a762d60057296a29e389d0e38cba81e . "./code.cg2js/cg-os-widget.fasl")
(#x9a68a960fe2fffd047f7b29a0bd51d76 . "./code.cg2js/cg-check-box.fasl")
(#x6817b389170cda97680eb079f686feac . "./code.cg2js/cg-choice-list.fasl")
(#xcbc9c4d39f3820715f37474d6cf36930 . "./code.cg2js/cg-layout.fasl")
(#xf313f26e4948180edd4bc7b3b4ea0852 . "./code.cg2js/cg-group-box.fasl")
(#x11e1aff1e86d69cc0a3eaf271eadf077 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x76b4cb7eb525b2ac2bc5767d35dbf891 . "./code.cg2js/cg-radio-button.fasl")
(#x8461d859a23b51e83f00c1e8ef89e5f9 . "./code.cg2js/cg-tab-control.fasl")
(#xb69e6ed14ab18f43daedb98259fce9d7 . "./code.cg2js/cg.fasl")
(#x8c945dbf66f1d7abf99bf837a6f5c304 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xfc29bbfdb7c7b4ad1a7034729b8e7d16 . "./code.cg2js/cg-dropping-outline.fasl")
(#xa116d94779de7391b1ee10e14815bb0c . "./code.cg2js/cg-get-pixmap.fasl")
(#x3a56a4a160e90f93da719cb3b0f12a94 . "./code.cg2js/cg-grid-widget.fasl")
(#xda3311c308fd7b02194cb243967e461c . "./code.cg2js/cg-utility-dialog.fasl")
(#x757f49b9176ad4c009c61f1f6bd132f8 . "./code.cg2js/cg-static-text.fasl")
(#xcf587d4d6dc58b35467b461c29895f5f . "./code.cg2js/cg-class-slot-grid.fasl")
(#x593d7c9dfba142f710fee37b5ae56dcb . "./code.cg2js/cg-object-editor-layout.fasl")
(#xcd232e4fa317ef439c97fdd4dcf9e932 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x3b73e92119794fc1439bc96844081d03 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x3c3cbd6294574dde9ed212576f598dc6 . "./code.cg2js/cg-plot-widget.fasl")
(#x8e785d5de63cd3469d813e943ea9d06e . "./code.cg2js/cg-color-dialog.fasl")
(#xc2efabb90588e1c9909b9f119be10abf . "./code.cg2js/cg-icon-pixmap.fasl")
(#x25c4e6b613900237cc279e529be33754 . "./code.cg2js/cg-icon.fasl")
(#x69947df5c3d0bfac86faaf06e1d69a9a . "./code.cg2js/cg-dev.fasl")
(#x102f40eae85c2afb96a0165a6d79407e . "./code.cg2js/cg-choose-printer.fasl")
(#x02cd3a4738bbf074441c8be70d44f868 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0351de2da9e30fd622deca474ab42ac2 . "./code.cg2js/cg-menu.fasl")
(#x31cfab89de39604a86585fd61d701640 . "./code.cg2js/cg-yes-no-list.fasl")
(#x0aecaa0c26308b40e671fd6f240d228c . "./code.cg2js/cg-palette.fasl")
(#xd6f43e9105ac29ad322c8e8313b8f280 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xf6720cf3a4f71decdd083dac53d4db0c . "./code.cg2js/cg-message-dialog.fasl")
(#x7e498bdc28517e2a43acf4ff44975b5b . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb729f48c922232ef500f428190c32630 . "./code.cg2js/cg-split-bar.fasl")
(#x31fb5ba5c564e942df3e656883c231d9 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xeef0be7fc4f72756448b129d75a770e0 . "./code.cg2js/cg-web-browser.fasl")
(#xbc3da0b0976f2e9b82c85f758cfd4d1d . "./code.cg2js/cg-project-window.fasl")
(#x5f4d973aeb3b3b089aa1e6f10ae6b7f0 . "./code.cg2js/cg-picture-widget.fasl")
(#x53a26db1540bb6d4952869cde8b7589a . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xffff477a545270ce492c7b058d495143 . "./code.cg2js/cg-object-editor.fasl")
(#xbf6feb8de2ad81e1e10285ac387e71d4 . "./code.cg2js/cg-paren-matching.fasl")
(#x139e44a98437062087ad397b8a4a869a . "./code.cg2js/cg-sample-file-menu.fasl")
(#x64c83500452781f59dbdeacfb4601367 . "./code.cg2js/cg-drawable.fasl")
(#xc88dd69b5d53324cd7808e14c483b354 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4e002d4e4feff5a1db806ab6ff94ae42 . "./code.cg2js/cg-html-widget.fasl")
(#xef5b0b2142a45966337c4dc042bbde93 . "./code.cg2js/cg-curve.fasl")
(#x1c9fb0fdd5d866aebb77c498412aae82 . "./code.cg2js/cg-yes-no-string.fasl")
(#x97433f73037555a2b43347f2b30d1953 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9324c9b30c1c3c93e8e35f0afc5fa349 . "./code.cg2js/cg-html-dialog.fasl")
(#x90657b7ead8c71ca8a54012681b8c3d5 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x83882a623215b7adab7eb42ca1eac87b . "./code.cg2js/cg-tray.fasl")
(#x6bcd837396ee5fedd3f55fa4f540d035 . "./code.idejs/ide-apropos.fasl")
(#x2acb8317e6c8a52c249a3ed4620122e2 . "./code.idejs/ide-window-list.fasl")
(#xb1b4b197b6be75fb76bfee907b8a371e . "./code.idejs/ide-class-browser.fasl")
(#xbfafae3520dd8c2c2f84eee97deb68d1 . "./code.idejs/ide-base.fasl")
(#x6bde91639de2a1ba9538a596c8e9092f . "./code.idejs/ide-process-browser.fasl")
(#x7fc515513070322b785d114952cd55e0 . "./code.idejs/ide-acache.fasl")
(#x791b50850da7c477ac73cd4850243b84 . "./code.idejs/ide-navigator.fasl")
(#x09c7c5a0c25cf1bcf7bba9902139c943 . "./code.idejs/ide-trace.fasl")
(#x3f81df8b2f7a414448ce0e3662dbcce9 . "./code.idejs/ide-project.fasl")
(#xb626fd74411fdaf84abe4c4a7e33e5e5 . "./code.idejs/ide-patch-dialog.fasl")
(#x88648921b2761387c84d31f4fb2d2cc6 . "./code.idejs/ide-grapher.fasl")
(#x85f851a31b2e727faaf4f941133a375e . "./code.idejs/ide-dev.fasl")
(#x3ade6c56c9fe6b6dac8198ec6677fb03 . "./code.idejs/ide-keyshort.fasl")
(#xff6294f4af984044b0585f895d28bc7c . "./code.idejs/ide-editor.fasl")
(#x1041aad45b691f9d9fd065bd90bd1f87 . "./code.idejs/ide-inspector.fasl")
(#x0c331db3c2061473cf1b31d7a63e3961 . "./code.idejs/ide.fasl")
(#x09777e4a52bea591e4286335cba90ab2 . "./code.idejs/ide-clipboard.fasl")
(#xb95799c11493f8754dc92bd267cd4e25 . "./code.idejs/ide-find-definitions.fasl")
(#x6e74f6403f43c62940687881455c0b8d . "./code.idejs/ide-form.fasl")
(#xc9608a5df8c198c77317e749b7bbb657 . "./code.idejs/ide-intro.fasl")
(#xd1e44352395c0126bb413790f7a1ec69 . "./code.idejs/ide-find-in-files.fasl")
(#xf61336d9581eca2676c5ffaea312b4b6 . "./code.idejs/ide-package-list.fasl")
(#x793b58d4b29e45dfa2e3036e16e20da3 . "./code.idejs/ide-class-grapher.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
